{ RRRRRR                  ReportBuilder Class LmySQLrary                  BBBBB
  RR   RR                                                              BB   BB
  RRRRRR                 Digital Metaphors Corporation                 BB BB
  RR  RR                                                               BB   BB
  RR   RR                   Copyright (c) 1996-2002                    BBBBB   }

unit daMySQLDAC;

interface

{$I ppIfDef.pas}

uses Classes, SysUtils, Forms, ExtCtrls,  DB, Contnrs,
     ppClass, ppComm, ppDBPipe, ppDB, ppClasUt, ppTypes,
     daDB, daQueryDataView, daDataView, daPreviewDataDlg,
     mySQLDbTables{mySQLDatabase, mySQLStoredProc, mySQLQuery, mySQLCustomDataSet, mySQLTable};



type

  {Interbase Express (mySQL) DataView Classes:

     1.  mySQL TDataSet descendants
           - TDataSets that can be children of a DataView.
           - Override the HasParent method of TComponent to return True
           - Must be registerd with the Delphi IDE using the RegisterNoIcon procedure

       a. TdaChildmySQLQuery - TmySQLQuery descendant that can be a child of a DataView
       b. TdaChildmySQLTable - TmySQLTable descendant that can be a child of a DataView
       c. TdaChildmySQLStoredProc - TmySQLStoredProc descendant that can be a child of a DataView

     2.  TdamySQLSession
           - descendant of TppSession
           - implements GetDatabaseNames, GetTableNames, etc.

     3.  TdamySQLDataSet
          - descendant of TppDataSet
          - implements GetFieldNames for SQL

     4.  TdamySQLQueryDataView
          - descendant of TppQueryDataView
          - uses the above classes to create the required
            Query -> DataSource -> Pipeline -> Report connection
          - uses the TdaSQL object built by the QueryWizard to assign
            SQL to the TmySQLQuery etc.

      }

  { TdaChildmySQLQuery }
  TdaChildmySQLQuery = class(TmySQLQuery)
    public
      function HasParent: Boolean; override;
    end;  {class, TdaChildmySQLQuery}

  { TdaChildmySQLTable }
  TdaChildmySQLTable = class(TmySQLTable)
    public
      function HasParent: Boolean; override;
    end;  {class, TdaChildmySQLTable}

  { TdaChildmySQLStoredProc }
  {TdaChildmySQLStoredProc = class(TmySQLStoredProc)
    public
      function HasParent: Boolean; override;
    end;}  {class, TdaChildmySQLStoredProc}


  { TdamySQLSession }
  TdamySQLSession = class(TdaSession)
    private
      FReservedWords: TStringList;
      FFunctions: TStringList;
      procedure BuildReservedWordList;
      procedure BuildFunctionList;

    protected
      function GetDefaultDatabase(const aDatabaseName: String): TComponent; override;
      function IsNamedDatabase(const aDatabaseName: String; aDatabase: TComponent): Boolean; override;

    public
      constructor Create(aComponent: TComponent); override;
      destructor Destroy; override;
      class function ClassDescription: String; override;
      class function DataSetClass: TdaDataSetClass; override;
      class function DatabaseClass: TComponentClass; override;
      class function GetDefaultmySQLDatabase: TmySQLDatabase;

      function  ContainsSQLFunctionCall(const aString: String; aDatabaseType: TppDatabaseType): Boolean; override;
      function  IsSQLReservedWord(const aString: String; aDatabaseType: TppDatabaseType): Boolean; override;
      function  GetSearchCriteriaDateFormat(aDatabaseType: TppDatabaseType; const aDatabaseName: String): String; override;
      function  GetSearchCriteriaTimeFormat(aDatabaseType: TppDatabaseType; const aDatabaseName: String): String; override;

       procedure GetDatabaseNames(aList: TStrings); override;
      function  GetDatabaseType(const aDatabaseName: String): TppDatabaseType; override;
      procedure GetTableNames(const aDatabaseName: String; aList: TStrings); override;
      function  ValidDatabaseTypes: TppDatabaseTypes; override;

  end; {class, TdamySQLSession}



  { TdamySQLDataSet }
  TdamySQLDataSet = class(TdaDataSet)
    private
      FQuery: TmySQLQuery;

      function GetQuery: TmySQLQuery;

    protected
      procedure BuildFieldList; override;
      function  GetActive: Boolean; override;
      procedure SetActive(Value: Boolean); override;
      procedure SetDatabase(aDatabase: TComponent); override;
      procedure SetDataName(const aDataName: String); override;

      property Query: TmySQLQuery read GetQuery;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function ClassDescription: String; override;

      procedure GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings); override;
      procedure GetFieldsForSQL(aList: TList; aSQL: TStrings); override;

  end; {class, TdamySQLDataSet}


  { TdamySQLQueryDataView }
  TdamySQLQueryDataView = class(TdaQueryDataView)
    private
      FDataSource: TppChildDataSource;
      FQuery: TdaChildmySQLQuery;

    protected
      procedure SQLChanged; override;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function PreviewFormClass: TFormClass; override;
      class function SessionClass: TClass; override;

      procedure Init; override;
      procedure ConnectPipelinesToData; override;

    published
      property DataSource: TppChildDataSource read FDataSource;

  end; {class, TdamySQLQueryDataView}


  {Delphi design time registration}
  procedure Register;


implementation

const
  cDefaultDatabase = 'DefaultmySQLDatabase';

var
  umySQLDatabase: TmySQLDatabase;
  //umySQLTransaction: TmySQLTransaction;


{******************************************************************************
 *
 ** R E G I S T E R
 *
{******************************************************************************}

procedure Register;
begin

  {mySQL DataAccess Components}
  RegisterNoIcon([TdaChildmySQLQuery, TdaChildmySQLTable{, TdaChildmySQLStoredProc}]);

  {mySQL DataViews}
  RegisterNoIcon([TdamySQLQueryDataView]);

end;


{******************************************************************************
 *
 ** C H I L D   I B  D A T A   A C C E S S   C O M P O N  E N T S
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaChildmySQLQuery.HasParent }

function TdaChildmySQLQuery.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{------------------------------------------------------------------------------}
{ TdaChildmySQLTable.HasParent }

function TdaChildmySQLTable.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}


{------------------------------------------------------------------------------}
{ TdaChildmySQLStoredProc.HasParent }
{
function TdaChildmySQLStoredProc.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}
  

{******************************************************************************
 *
 ** I B   S E S S I O N
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdamySQLSession.Create }

constructor TdamySQLSession.Create(aComponent: TComponent);
begin
  inherited Create(aComponent);

  FFunctions := TStringList.Create;
  FReservedWords := TStringList.Create;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdamySQLSession.Destroy }

destructor TdaMySQLSession.Destroy;
begin
  FFunctions.Free;
  FReservedWords.Free;

  inherited Destroy;

end; {destructor, Destroy}
{------------------------------------------------------------------------------}
{ TdaMySQLSession.IsSQLReservedWord }

function TdaMySQLSession.IsSQLReservedWord(const aString: String; aDatabaseType: TppDatabaseType): Boolean;
begin

  if (FReservedWords.Count = 0) then
    BuildReservedWordList;

  Result := (FReservedWords.IndexOf(aString) <> -1);

end; {procedure, IsSQLReservedWord}

{------------------------------------------------------------------------------}
{ TdaMySQLSession.BuildReservedWordList }

procedure TdaMySQLSession.BuildReservedWordList;
begin
  FReservedWords.CommaText := 'action,add,aggregate,all,alter,'+
  'after,and,as,asc,avg,avg_row_length,'+
  'auto_increment,between,bigint,bit,binary,blob,bool,both,by,'+
	'cascade,case,char,character,change,'+
  'check,checksum,column,columns,comment,'+
  'constraint,create,cross,current_date,current_time,current_timestamp,'+
	'data,database,databases,date,datetime,'+
  'day,day_hour,day_minute,day_second,'+
  'dayofmonth,dayofweek,dayofyear,dec,decimal,default,delayed,'+
	'delay_key_write,delete,desc,describe,'+
  'distinct,distinctrow,double,drop,end,'+
  'else,escape,escaped,enclosed,enum,explain,'+
  'exists,fields,file,first,float,'+
	'float4,float8,flush,foreign,'+
	'from,for,full,function,'+
	'global,grant,grants,group,'+
	'having,heap,high_priority,hour,'+
	'hour_minute,hour_second,hosts,identified,'+
	'ignore,in,index,infile,'+
	'inner,insert,insert_id,int,'+
	'integer,interval,int1,int2,'+
	'int3,int4,int8,into,'+
	'if,is,isam,join,'+
	'key,keys,kill,last_insert_id,'+
	'leading,left,length,like,'+
	'lines,limit,load,local,'+
	'lock,logs,long,longblob,'+
	'longtext,low_priority,max,max_rows,'+
	'match,mediumblob,mediumtext,mediumint,'+
	'middleint,min_rows,minute,minute_second,'+
	'modify,month,monthname,myisam,'+
	'natural,numeric,no,not,'+
	'null,on,optimize,option,'+
	'optionally,or,order,outer,'+
	'outfile,pack_keys,partial,password,'+
	'precision,primary,procedure,process,'+
	'processlist,privileges,read,real,'+
	'references,reload,regexp,rename,'+
	'replace,restrict,returns,revoke,'+
	'rlike,row,rows,second,'+
	'select,set,show,shutdown,'+
	'smallint,soname,sql_big_tables,sql_big_selects,'+
	'sql_low_priority_updates,sql_log_off,sql_log_update,sql_select_limit,'+
	'sql_small_result,sql_big_result,sql_warnings,straight_join,'+
	'starting,status,string,table,'+
	'tables,temporary,terminated,text,'+
	'then,time,timestamp,tinyblob,'+
	'tinytext,tinyint,trailing,to,'+
	'type,use,using,unique,'+
	'unlock,unsigned,update,usage,'+
	'values,varchar,variables,varying,'+
	'varbinary,with,write,when,'+
	'where,year,year_month,zerofill';
end; {procedure, BuildReservedWordList}

{------------------------------------------------------------------------------}
{ TdaMySQLSession.ContainsSQLFunctionCall }

function TdaMySQLSession.ContainsSQLFunctionCall(const aString: String; aDatabaseType: TppDatabaseType): Boolean;
var
  liIndex: Integer;
  liPosition: Integer;
  lsFunction: String;
begin

  if (FFunctions.Count = 0) then
    BuildFunctionList;

  liIndex := 0;
  liPosition := 0;
  lsFunction := UpperCase(aString);

  while (liIndex < FFunctions.Count) and (liPosition = 0) do
    begin
      liPosition := Pos(FFunctions[liIndex], lsFunction);

      Inc(liIndex);
    end;

  Result := (liPosition <> 0);

end; {procedure, ContainsSQLFunctionCall}

{------------------------------------------------------------------------------}
{ TdaMySQLSession.BuildFunctionList }

procedure TdaMySQLSession.BuildFunctionList;
begin
  {aggregate functions}
  FFunctions.CommaText := 'ABS(,ACOS(,ADDDATE(,ASCII(,ASIN(,ATAN(,ATAN(,AVG(,BENCHMARK(,BIN(,BIT_AND(,BIT_COUNT(,BIT_OR(,CEILING(,CHAR(,CHAR_LENGTH(,CHARACTER_LENGTH(,'+
	'COALESCE(,CONCAT(,CONV(,COS(,COT(,COUNT(,CURDATE(,CURRENT_DATE,CURRENT_TIME,CURRENT_TIMESTAMP,CURTIME(,DATABASE(,DATE_ADD(,DATE_FORMAT(,'+
	'DATE_SUB(,DAYNAME(,DAYOFMONTH(,DAYOFWEEK(,DAYOFYEAR(,DECODE(,DEGREES(,ELT(,ENCODE(,ENCRYPT(,EXP(,EXPORT_SET(,EXTRACT(,FIELD(,'+
	'FIND_IN_SET(,FLOOR(,FORMAT(,FROM_DAYS(,FROM_UNIXTIME(,GET_LOCK(,GREATEST(,HEX(,HOUR(,IF(,IFNULL(,INSERT(,INSTR(,INTERVAL(,ISNULL(,LAST_INSERT_ID(,'+
	'LCASE(,LEAST(,LEFT(,LENGTH(,LOCATE(,LOG(,LOWER(,LPAD(,LTRIM(,MAKE_SET(,MAX(,MD(,MID(,MIN(,MINUTE(,MOD(,MONTH(,MONTHNAME(,NOW(,OCT(,OCTET_LENGTH(,ORD(,PASSWORD(,PERIOD_ADD(,'+
	'PERIOD_DIFF(,PI(,POSITION(,POW(,POWER(,QUARTER(,RADIANS(,RAND(,RELEASE_LOCK(,REPEAT(,REPLACE(,REVERSE(,RIGHT(,ROUND(,RPAD(,RTRIM(,SEC_TO_TIME(,'+
	'SECOND(,SESSION_USER(,SIGN(,SIN(,SOUNDEX(,SPACE(,SQRT(,STD(,STDDEV(,STRCMP(,SUBDATE(,SUBSTRING(,SUBSTRING_INDEX(,SUM(,SYSDATE(,SYSTEM_USER(,'+
	'TAN(,TIME_FORMAT(,TIME_TO_SEC(,TO_DAYS(,TRIM(,TRUNCATE(,UCASE(,UNIX_TIMESTAMP(,UPPER(,USER(,VERSION(,WEEK(,WEEKDAY(,YEAR(';
end; {procedure, BuildFunctionList}

{------------------------------------------------------------------------------}
{ TdaMySQLSession.GetSearchCriteriaDateFormat }

function TdaMySQLSession.GetSearchCriteriaDateFormat(aDatabaseType: TppDatabaseType; const aDatabaseName: String): String;
begin
  Result := 'YYYY-MM-DD';
end; {function, GetSearchCriteriaDateFormat}

{------------------------------------------------------------------------------}
{ TdaMySQLSession.GetSearchCriteriaTimeFormat }

function TdaMySQLSession.GetSearchCriteriaTimeFormat(aDatabaseType: TppDatabaseType; const aDatabaseName: String): String;
begin
  Result := 'HH:MM:SS';
end; {function, GetSearchCriteriaTimeFormat}
{------------------------------------------------------------------------------}
{ TdamySQLSession.ClassDescription }

class function TdamySQLSession.ClassDescription: String;
begin
  Result := 'mySQLSession';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdamySQLSession.DataSetClass }

class function TdamySQLSession.DataSetClass: TdaDataSetClass;
begin
  Result := TdamySQLDataSet;
end; {class function, DataSetClass}

{------------------------------------------------------------------------------}
{ TdamySQLSession.DatabaseClass }

class function TdamySQLSession.DatabaseClass: TComponentClass;
begin
  Result := TmySQLDatabase;
end;


{------------------------------------------------------------------------------}
{ TdamySQLSession.GetDefaultmySQLDatabase }

class function TdamySQLSession.GetDefaultmySQLDatabase: TmySQLDatabase;
begin

  {create the default mySQL database, if needed}
  if (umySQLDatabase = nil) then
    begin
      {create default mySQL database}
      umySQLDatabase := TmySQLDatabase.Create(nil);
      umySQLDatabase.Name := cDefaultDatabase;

      {create a default transaction for the mySQLDatabase}
      //umySQLTransaction := TmySQLTransaction.Create(nil);

      //umySQLDatabase.DefaultTransaction := umySQLTransaction;

    end;

  Result := umySQLDatabase;
  
end; {function, daGetDefaultmySQLDatabase}


{------------------------------------------------------------------------------}
{ TdamySQLSession.GetTableNames }

procedure TdamySQLSession.GetTableNames(const aDatabaseName: String; aList: TStrings);
var
  lDatabase: TmySQLDatabase;
  lTable: TmySQLTable;

begin
  {get the database}
  lDatabase := TmySQLDatabase(GetDatabaseForName(aDatabaseName));

  lTable := TmySQLTable.Create(nil);
  lTable.Database := lDatabase;

  {connection must be active to get table names}
  if not lDatabase.Connected then
    lDatabase.Connected := True;

  {get list of table names from a table object}
  if lDatabase.Connected then
    //aList.Assign(lTable.TableNames);

  lTable.Free;

end; {procedure, GetTableNames}

{------------------------------------------------------------------------------}
{ TdamySQLSession.GetDatabaseNames }

procedure TdamySQLSession.GetDatabaseNames(aList: TStrings);
begin

  {call inherited to build list of avaailable TmySQLDatabase components}
  inherited GetDatabaseNames(aList);

  {could add hard-coded connection strings here, or could
   read from an .ini file }

end; {procedure, GetDatabaseNames}

{------------------------------------------------------------------------------}
{ TdamySQLSession.GetDefaultDatabase }

function TdamySQLSession.GetDefaultDatabase(const aDatabaseName: String): TComponent;
var
  lmySQLDatabase: TmySQLDatabase;
begin

  lmySQLDatabase := GetDefaultmySQLDatabase;

  {set DatabaseName property, if needed}
  if (lmySQLDatabase.DatabaseName <> aDatabaseName) then
    begin

      if lmySQLDatabase.Connected then
        lmySQLDatabase.Connected := False;

      lmySQLDatabase.DatabaseName := aDatabaseName;

    end;

  Result := lmySQLDatabase;

end; {function, GetDefaultDatabase}

{------------------------------------------------------------------------------}
{ TdamySQLSession.IsNamedDatabase }

function TdamySQLSession.IsNamedDatabase(const aDatabaseName: String; aDatabase: TComponent): Boolean;
begin

  Result := (AnsiCompareText(aDatabase.Name, aDatabaseName) = 0) or
            (AnsiCompareText(TmySQLDatabase(aDatabase).DatabaseName, aDatabaseName) = 0);

end; {function, IsNamedDatabase}

{------------------------------------------------------------------------------}
{ TdamySQLSession.ValidDatabaseTypes }

function TdamySQLSession.ValidDatabaseTypes: TppDatabaseTypes;
begin
  Result := [dtInterBase];
end; {procedure, ValidDatabaseTypes}

{------------------------------------------------------------------------------}
{ TdamySQLSession.GetDatabaseType }

function TdamySQLSession.GetDatabaseType(const aDatabaseName: String): TppDatabaseType;
begin
  Result := dtInterBase;
end; {function, GetDatabaseType}


{******************************************************************************
 *
 ** I B  D A T A S E T
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.Create }

constructor TdamySQLDataSet.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  FQuery := nil;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.Destroy }

destructor TdamySQLDataSet.Destroy;
begin

  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.ClassDescription }

class function TdamySQLDataSet.ClassDescription: String;
begin
  Result := 'mySQLDataSet';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.GetActive }

function TdamySQLDataSet.GetActive: Boolean;
begin
  Result := GetQuery.Active
end; {function, GetActive}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.SetActive }

procedure TdamySQLDataSet.SetActive(Value: Boolean);
begin
  GetQuery.Active := Value;
end; {procedure, SetActive}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.GetQuery }

function TdamySQLDataSet.GetQuery: TmySQLQuery;
begin

  {create mySQL table, if needed}
  if (FQuery = nil) then
    FQuery := TmySQLQuery.Create(Self);

  Result := FQuery;

end; {procedure, GetQuery}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.SetDatabase }

procedure TdamySQLDataSet.SetDatabase(aDatabase: TComponent);
begin

  inherited SetDatabase(aDatabase);

  {table cannot be active to set database property}
  if GetQuery.Active then
    FQuery.Active := False;

  {set mySQLQuery.Database}
  FQuery.Database := TmySQLDatabase(aDatabase);

end; {procedure, SetDatabase}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.SetDataName }

procedure TdamySQLDataSet.SetDataName(const aDataName: String);
begin

  inherited SetDataName(aDataName);

  {dataset cannot be active to set data name}
  if GetQuery.Active then
    FQuery.Active := False;

  {construct an SQL statment that returns an empty result set,
   this is used to get the field information }
  FQuery.SQL.Text := 'SELECT * FROM ' + aDataName +
                     ' WHERE ''c'' <> ''c'' ';

end; {procedure, SetDataName}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.BuildFieldList }

procedure TdamySQLDataSet.BuildFieldList;
var
  liIndex: Integer;
  lQueryField: TField;
  lField: TppField;
begin

  inherited BuildFieldList;

  {set table to active}
  if not(GetQuery.Active) then
    FQuery.Active := True;

  {create TppField objects for each field in the table}
  for liIndex := 0 to FQuery.FieldCount - 1 do
    begin
      lQueryField := FQuery.Fields[liIndex];

      lField := TppField.Create(nil);

      lField.TableName := DataName;
      lField.FieldName := lQueryField.FieldName;
      lField.FieldAlias := lQueryField.DisplayLabel;
      lField.FieldLength := lQueryField.Size;
      lField.DataType := ppConvertFieldType(lQueryField.DataType);
      lField.DisplayWidth := lQueryField.DisplayWidth;

      AddField(lField);
    end;

end; {function, BuildFieldList}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.GetFieldNamesForSQL }

procedure TdamySQLDataSet.GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings);
var
  lQuery: TmySQLQuery;
begin
  aList.Clear;

  {create a temporary mySQL query}
  lQuery := TmySQLQuery.Create(Self);

  {set the database and SQL properties}
  lQuery.Database := TmySQLDatabase(Database);
  lQuery.SQL := aSQL;

  {get the field names}
  lQuery.GetFieldNames(aList);

  lQuery.Free;

end; {procedure, GetFieldNamesForSQL}

{------------------------------------------------------------------------------}
{ TdamySQLDataSet.GetFieldsForSQL }

procedure TdamySQLDataSet.GetFieldsForSQL(aList: TList; aSQL: TStrings);
var
  lQuery: TmySQLQuery;
  lQueryField: TField;
  lField: TppField;
  liIndex: Integer;
begin
  aList.Clear;

  {create a temporary mySQL query}
  lQuery := TmySQLQuery.Create(Self);

  try

    {assign databae and SQL properties}
    lQuery.Database := TmySQLDatabase(Database);
    lQuery.SQL := aSQL;

    {set query to active}
    lQuery.Active := True;

    {create a TppField object for each field in the query}
    for liIndex := 0 to lQuery.FieldCount - 1 do
      begin
        lQueryField := lQuery.Fields[liIndex];

        lField := TppField.Create(nil);

        lField.FieldName := lQueryField.FieldName;
        lField.FieldAlias := lQueryField.DisplayLabel;
        lField.FieldLength := lQueryField.Size;
        lField.DataType := ppConvertFieldType(lQueryField.DataType);
        lField.DisplayWidth := lQueryField.DisplayWidth;

        aList.Add(lField);
      end;

  finally
    lQuery.Free;

  end;

end; {procedure, GetFieldsForSQL}




{******************************************************************************
 *
 ** I B  Q U E R Y   D A T A V I E W
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdamySQLQueryDataView.Create }

constructor TdamySQLQueryDataView.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  {notes: 1. must use ChildQuery, ChildDataSource, ChildPipeline etc.
          2. use Self as owner for Query, DataSource etc.
          3. do NOT assign a Name }

  FQuery := TdaChildmySQLQuery.Create(Self);

  FDataSource := TppChildDataSource.Create(Self);
  FDataSource.DataSet := FQuery;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdamySQLQueryDataView.Destroy }

destructor TdamySQLQueryDataView.Destroy;
begin
  FDataSource.Free;
  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdamySQLQueryDataView.SessionClass }

class function TdamySQLQueryDataView.SessionClass: TClass;
begin
  Result := TdamySQLSession;
end; {class function, SessionClass}

{------------------------------------------------------------------------------}
{ TdamySQLQueryDataView.PreviewFormClass }

class function TdamySQLQueryDataView.PreviewFormClass: TFormClass;
begin
  Result := TFormClass(GetClass('TdaPreviewDataDialog'));
end; {class function, PreviewFormClass}


{------------------------------------------------------------------------------}
{ TdamySQLQueryDataView.ConnectPipelinesToData }

procedure TdamySQLQueryDataView.ConnectPipelinesToData;
begin

  if DataPipelineCount = 0 then Exit;

  {need to reconnect here}
  TppDBPipeline(DataPipelines[0]).DataSource := FDataSource;


end; {procedure, ConnectPipelinesToData}

{------------------------------------------------------------------------------}
{ TdamySQLQueryDataView.Init }

procedure TdamySQLQueryDataView.Init;
var
  lDataPipeline: TppChildDBPipeline;

begin

  inherited Init;

  if DataPipelineCount > 0 then Exit;

  {note: DataView's owner must own the DataPipeline }
  lDataPipeline := TppChildDBPipeline(ppComponentCreate(Self, TppChildDBPipeline));
  lDataPipeline.DataSource := FDataSource;
 
  lDataPipeline.AutoCreateFields := False;

  {add DataPipeline to the dataview }
  lDataPipeline.DataView := Self;

end; {procedure, Init}

{------------------------------------------------------------------------------}
{ TdamySQLQueryDataView.SQLChanged }

procedure TdamySQLQueryDataView.SQLChanged;
var
  lDatabase: TComponent;
begin

  if FQuery.Active then
    FQuery.Close;

  lDatabase := Session.GetDatabaseForName(SQL.DatabaseName);

  if (lDatabase = nil) then
    raise EDataError('TdamySQLQueryDataView.SQLChanged: No mySQLDatabase object found, ' + SQL.DatabaseName);

  FQuery.Database := TmySQLDatabase(lDatabase);
  FQuery.SQL := SQL.MagicSQLText;

end; {procedure, SQLChanged}

{******************************************************************************
 *
 ** P R O C E D U R E S   A N D   F U N C T I O N S
 *
{******************************************************************************}

initialization

  {register the mySQL descendant classes}
  RegisterClasses([TdaChildmySQLQuery, TdaChildmySQLTable{, TdaChildmySQLStoredProc}]);

  {register DADE descendant session, dataset, dataview}
  daRegisterSession(TdamySQLSession);
  daRegisterDataSet(TdamySQLDataSet);
  daRegisterDataView(TdamySQLQueryDataView);

  {initialize internal reference variables}
  umySQLDatabase := nil;
  //umySQLTransaction := nil;


finalization

  {free the default database object}
  umySQLDatabase.Free;
  umySQLDatabase := nil;
  //umySQLTransaction.Free;
  //umySQLTransaction := nil;

  {unregister the mySQL descendant classes}
  UnRegisterClasses([TdaChildmySQLQuery, TdaChildmySQLTable{, TdaChildmySQLStoredProc}]);

  {unregister DADE descendant the session, dataset, dataview}
  daUnRegisterSession(TdamySQLSession);
  daUnRegisterDataSet(TdamySQLDataSet);
  daUnRegisterDataView(TdamySQLQueryDataView);


end.
